open Geom_ops
let () = Graphics.open_graph " 500x500"

let offset = 100., 100.
let scale = 200

let transform offset v =
  let n = float_of_int (Q.(to_int (v * Q.of_int scale))) in
  int_of_float (offset +. n)

let vertex_position (x, y) =
  let (ox, oy) = offset in
  transform ox x, transform oy y

let draw_line (L ({ x = lx; y = ly }, V(vx, vy))) =
  let open Graphics in
  set_color black;
  let x, y = vertex_position (lx, ly) in
  moveto x y;
  let x', y' = vertex_position Q.(vx + lx, vy + ly) in
  lineto (x') (y');
  fill_circle x y 3

let draw_point ?(col=Graphics.red) ({x; y}) =
  let open Graphics in
  set_color col;
  let x, y = vertex_position (x, y) in
  fill_circle x y 4

let draw_segment (S ({ x = x1; y= y1 }, { x = x2; y = y2 })) =
  let open Graphics in
  let x, y = vertex_position (x1, y1) in
  moveto x y;
  set_color green;
  fill_circle x y 3;
  let x', y' = vertex_position (x2, y2) in
  set_color black;
  lineto (x') (y');
  set_color blue;
  fill_circle x' y' 3

let draw_polygon ?(col=Graphics.red) l =
  let a = Array.of_list (List.map (fun ({ x = vx; y = vy}) -> vertex_position (vx, vy)) l) in
  Graphics.set_color col;
  Graphics.fill_poly a;
  Graphics.set_color Graphics.black;
  Graphics.draw_poly a;
  List.iter (draw_point ~col) l
