(** *)
open Core.Std

let () = Show.init ()

let rec show solution problem id =
  match Show.wait_solution_et_probleme solution problem
          (sprintf "Displaying: %i" id)
          ([ 'q', ("quit", `Quit);
             'n', ("next", `Next);
             'd', ("delaunay", `Delaunay);
             'r', ("solution in destination", `Dest);
             's', ("skeleton", `Skeleton);
             '+', ("scale +", `Scale(2.));
             '-', ("scale -", `Scale(0.5));
           ])
  with
  | `Quit -> exit 0
  | `Next -> ()
  | `Delaunay -> Show.delaunay := (!Show.delaunay + 1) mod 4; show solution problem id
  | `Skeleton -> Show.skeleton := not !Show.skeleton; show solution problem id
  | `Scale(f) -> Show.scale_user := f *. !Show.scale_user; show solution problem id
  | `Dest -> Show.show_sol_destination := not (!Show.show_sol_destination); show solution problem id


let check problems file (solution:Http_api.solution)
    good_resemblance =
  let problem = Int.Table.find_exn problems solution.problem_id in
  let (shape,_) as prob = Http_api.readProblem problem in
  let resemblance = Solution.compute_resemblance shape solution.solution in
  match resemblance with
  | Error r ->
    Format.eprintf "Can't compute resemblance for %s pb:%i:@.%s@."
      file solution.problem_id (Error.to_string_hum r)
  | Ok s when s <> good_resemblance ->
    Format.eprintf "Wrong resemblance for %s pb:%i : %f instead of %f @."
      file solution.problem_id s good_resemblance;
    show solution.solution prob solution.problem_id
  | _ ->
    Format.eprintf "Good resemblance for %s pb:%i : %f@."
      file solution.problem_id good_resemblance

let () =
  let problems = Http_api.listProblems () in
  let problems = match Int.Table.create_mapped
                  problems
                  ~get_key:(fun (x:Http_api.problem) -> x.problem_id)
                  ~get_data:(fun x -> x) with
  | `Duplicate_keys _ -> invalid_argf "duplicate keys" ()
  | `Ok h -> h in
  Http_api.foldSolutions_sent ~init:()
    ~f:(fun () file s ->
        match s.Http_api.resemblance with
        | `Good f -> check problems file s f
        | _ -> ());
  (* Http_api.save_db "convex solver" *)
