(* #use "load.ml";; *)
open Core.Std
open Quompare

let myhull v =
  Geom_ops.hull v

(* let myhull v = v *)

(* crade et quadratique... *)
let diameter vert =
  let open Geom_ops in
  let hull = myhull vert in
  let first =
    match hull with
    | h1 :: h2 :: _ -> (h1, h2), norm2 (h1 -= h2)
    | _ -> assert false
  in
  List.fold_left hull ~init:first
    ~f:(fun acc e ->
       List.fold_left hull ~init:acc
         ~f:(fun ((_, dsquare) as acc) e' ->
            let v = e' -= e in
            let norm = norm2 v in
            if norm > dsquare then
              (e, e'), norm
            else
              acc))

open Geom_ops

let symetrie (L(l,v)) p =
  let lp = p -= l in
  let nv = normal v in
  let coef = Q.(of_int 1 / norm2 v) in
  l += coef *& (scal lp v *& v -& scal lp nv *& nv)

let change_coordinate a a' b b' c =
  let ab = b -= a in
  let nab = normal ab in
  let a'b' = b' -= a' in
  let na'b' = normal a'b' in
  let ac = c -= a in
  let coef = Q.(of_int 1 / norm2 ab) in
  let tx = Q.(coef * scal ab ac) in
  let ty = Q.(coef * scal nab ac) in
  a' += ((tx *& a'b') +& (ty *& na'b'))

(* point à distance 1 de (0, 0), proche de la diagonale.
   voir pyth.ml *)
let magic_vertex =
  { Geom_ops.
    x = {Q.num = Z.of_int 159140519; den = Z.of_int 225058681};
    y = {Q.num = Z.of_int 159140520; den = Z.of_int 225058681} }

let magic_segment =
  S(p 0 0, magic_vertex)

let bounding_rect verts =
  match verts with
  | [] -> assert false
  | v1 :: verts ->
    let min_x =
      List.fold_left ~init:v1.x
        ~f:(fun min_x v -> Q.min min_x v.x)
        verts
    in
    let min_y =
      List.fold_left ~init:v1.y
        ~f:(fun min_y v -> Q.min min_y v.y)
        verts
    in
    let max_x =
      List.fold_left ~init:v1.x
        ~f:(fun max_x v -> Q.max max_x v.x)
        verts
    in
    let max_y =
      List.fold_left ~init:v1.y
        ~f:(fun max_y v -> Q.max max_y v.y)
        verts
    in
    (min_x, min_y), (max_x, max_y)

let taille_sans_rotation verts =
  let (min_x, min_y), (max_x, max_y) = bounding_rect verts in
  let sx = Q.(max_x - min_x) in
  let sy = Q.(max_y - min_y) in
  sx, sy

let fit_in_square verts =
  let sx, sy = taille_sans_rotation verts in
  sx <= Q.one && sy <= Q.one

let default_transfo = S(p 0 0,p 0 1), S(p 0 0, p 0 1)
let fitting_offset verts =
  let (min_x, min_y), (max_x, max_y) = bounding_rect verts in
  let off_x = Q.neg (Q.(neg (min min_x zero) + (min (one - max_x) zero))) in
  let off_y = Q.neg (Q.(neg (min min_y zero) + (min (one - max_y) zero))) in
  let moved = { x = off_x; y = off_y } in
  (* Format.printf "off %a@." Geom_ops.pp_vertex moved; *)
  S(moved, moved += v 0 1), S(p 0 0, p 0 1)

let bounding_offsets verts =
  let (min_x, min_y), (_max_x, _max_y) = bounding_rect verts in
  V(Q.neg (Q.min min_x Q.zero),
    Q.neg (Q.min min_y Q.zero))

let sin_carre ab cd =
  let pr = vect_prod ab cd in
  Q.(pr * pr / (norm2 ab * norm2 cd))

let repere_proche (S(a,b)) =
  let ab = b -= a in
  if norm2 ab = Q.of_int 2 then
    (* TODO: faire pout toutes les racines *)
    S(a,b), S(p 0 0, p 1 1)
  else
    let cd = v 1 0 in
    let (sin, cos) = Pyth.sin_of_sin_square (sin_carre ab cd) in
    let cos, sin =
      let V(x, y) = ab in
      (if x >= Q.zero then
         cos
       else
         Q.neg cos),
      (if y >= Q.zero then
         sin
       else
         Q.neg sin)
    in
    S(a, a += V(cos, sin)), magic_segment

let transform_vertices (S(a,b)) (S(a',b')) vert =
  List.map ~f:(change_coordinate a a' b b') vert

let transform_seg offset (S(a,b)) (S(a',b')) (S(v1,v2)) =
  S(change_coordinate a a' b b' v1 += offset,
    change_coordinate a a' b b' v2 += offset)

let deplace_dans_le_carre vert =
  let vert = myhull vert in
  let (diam1, diam2), _dist = diameter vert in
  let repere, dest =
    if fit_in_square vert then
      (* let () = Format.printf "fit@." in *)
      fitting_offset vert
    else
      (* let () = Format.printf "not fit@." in *)
      repere_proche (S(diam1, diam2))
  in
  let transformed = transform_vertices repere dest vert in
  let offset = bounding_offsets transformed in
  List.map ~f:(fun v -> v += offset) transformed,
  transform_seg offset repere magic_segment (S(diam1, diam2)),
  (repere, dest ++= offset)


let strictly_inside_hull_cw hull_cw v =
  forall_edges hull_cw
    ~f:(fun h1 h2 ->
        vect_prod (h1 -= h2) (h1 -= v) > Q.zero)

let trouve_hors_du_hull_cw hull_cw v =
  find_edge hull_cw
    ~f:(fun h1 h2 ->
        vect_prod (h1 -= h2) (h1 -= v) > Q.zero)

let trouve_coupage_suivant hull_cw pts =
  List.find_map pts ~f:(fun pt ->
      match trouve_hors_du_hull_cw hull_cw pt with
      | None -> None
      | Some (a,b) -> Some(pt, L(b,b -= a)))

let position v = v.Geom_a_histoire.position

let fait_un_pli (state:Pliage.state) (objective:vertex list) =
  let verts =
    List.join @@ List.map state.Pliage.surface (List.map ~f:position)
  in
  let current_hull = myhull verts in
  match trouve_coupage_suivant objective current_hull with
  | None -> None
  | Some (pt, l) ->
    Some (pt, l, Pliage.plie l state)

let fait_n_plis state probleme n =
  let objectif = hull probleme in
  let rec loop state hist n =
    if n = 0 then
      state, List.rev hist
    else
      match fait_un_pli state objectif with
      | None -> state, List.rev hist
      | Some (_pt, pli, state) ->
        loop state (pli :: hist) (n-1)
  in
  loop state [] n

let problem_vertices (polys, _) =
  List.dedup ~compare:compare_vertex (List.join polys)

let transform_hist_vert (S(a,b)) (S(a',b')) vert =
  List.map
    ~f:(fun v ->
        let position =
          change_coordinate a a' b b' v.Geom_a_histoire.position
        in
        { v with Geom_a_histoire.position })
    vert

let back_transform state (repere, dest) =
  let surface =
    List.map
      ~f:(transform_hist_vert dest repere)
      state.Pliage.surface
  in
  { Pliage.surface; plis = [] }

let launch_and_replace problem n =
  let objectif = problem_vertices problem in
  let tr_objectif,_,(repere, dest) = deplace_dans_le_carre objectif in
  let state, hist = fait_n_plis Pliage.carre tr_objectif n in
  back_transform state (repere, dest), hist

let solve_while_not_too_big problem =
  let objectif = problem_vertices problem in
  let tr_objectif,_,(repere, dest) = deplace_dans_le_carre objectif in
  let tr_objectif = hull tr_objectif in
  let r = ref 0 in
  let rec loop state hist n =
    if Int.(n <= 0) then
      state, hist
    else
      let () = incr r in
      match fait_un_pli state tr_objectif with
      | None -> state, hist
      | Some (_p, pli, new_state) ->
        let state' = back_transform new_state (repere, dest) in
        let sol = Pliage.to_solution state' in
        if Int.(Solution.solution_size sol >= 5000) then
          state, hist
        else
          loop new_state (pli :: hist) (n-1)
  in
  let result, hist = loop Pliage.carre [] 100 in
  Printf.printf "steps %i\n%!" !r;
  back_transform result (repere, dest), hist

(*******************)
(*    Testation    *)
(*******************)

(*
let problem_directory = "../Problems"
let problem_file id = Filename.concat problem_directory (Printf.sprintf "%04i.origami" id)
let parse_problem i = Icfp_parse.parse (In_channel.read_all (problem_file i))
let problem = parse_problem 11

let test m n =
  let problem = parse_problem m in
  let res = launch_and_replace problem n in
  let pro = Pliage.to_probleme res in
  let sol = Pliage.to_solution res in
  Show.wait_solution_et_probleme sol pro "humph" ['q', ("quit", ())]

let () =
  for i = 0 to 100 do
    test 80 i
  done;
  for i = 80 to 80 do
    for j = 1 to 10 do
      test i j
    done
  done
  (* test 11 0; *)
  (* test 11 1; *)
  (* test 11 2; *)
  (* test 13 0; *)
  (* test 13 1; *)
  (* test 13 2; *)
  (* test 13 3 *)

let obj = problem_vertices problem
let obj' = hull obj


let n1 = fait_n_plis Pliage.carre obj' 1
let pro = Pliage.to_probleme n1
let sol = Pliage.to_solution n1

let () =
  Show.wait_solution_et_probleme sol pro "humph" ['q', ("quit", ())]

let p1 = p 0 0
let p2 = p 1 0
let p3 = p 1 1
let p4 = p 0 1

let pts = [p1;p2;p3;p4]
let () = assert(not (ccw (hull [p1;p2;p3;p4])))

let pp1 = pf 0.5 0.5
let pp2 = pf 0.5 (0.01)
let pp3 = pf 1.5 0.5

let () =
  Printf.printf "%b %b %b\n%!"
    (strictly_inside_hull_cw pts pp1)
    (strictly_inside_hull_cw pts pp2)
    (strictly_inside_hull_cw pts pp3)

let points =
  let () = Random.self_init () in
  List.map ~f:(fun p -> p += v 1 1) @@
  Array.to_list @@
  Array.init 50
    (fun _ ->
       { x = Q.of_float (Random.float 1.);
         y = Q.of_float (Random.float 1.) })

let p', diam_dest, (rep, dest) = deplace_dans_le_carre points
let (diam1, diam2), _dist = diameter points

let rec wait () =
  let { Graphics.key; Graphics.keypressed } = Graphics.(wait_next_event [ Key_pressed ]) in
  if keypressed && key = 'q' then ()
  else wait ()

let () = assert(not (ccw (hull points)))

let () =
  Graphics.clear_graph ();
  List.iter ~f:(Affichage_test.draw_point ~col:Graphics.blue) points;
  List.iter ~f:(Affichage_test.draw_point ~col:Graphics.red) p';
  List.iter ~f:Affichage_test.draw_segment
    [S(p 0 0, p 0 1);
     S(p 0 1, p 1 1);
     S(p 1 1, p 1 0);
     S(p 1 0, p 0 0);
     (* S(diam1, diam2); *)
     (* diam_dest; *)
     rep;
     dest;
    ];
  wait ()
*)
