#use "topfind";;
#require "curl";;
#require "unix";;
#require "ezjsonm";;

(* Liste des constantes *)
let problem_directory = "./Problems"
let key = "74-b522506ca82b646082cb05aeb9702281"
let hello_world = "http://2016sv.icfpcontest.org/api/hello"
let contest_status = "http://2016sv.icfpcontest.org/api/snapshot/list"
let contest_datas blob = "http://2016sv.icfpcontest.org/api/blob/" ^ blob

(* Callback pour écrire dans un buffer *)
let writer_callback a d =
  Buffer.add_string a d;
  String.length d

(* Fonction faisant une requete via l'API fournie *)
(* API requests are rate-limited. You must wait at least 1 second between successive API requests.
   Also, every API have their own rate limits, as described below in this page.
   When you exceed these limits, your requests will fail with 429 Rate limit exceeded. *)
let getIcfpcDatas url =
  let t = Curl.init () in
  let buff = Buffer.create 16384 in
  Curl.setopt t (Curl.CURLOPT_ENCODING Curl.CURL_ENCODING_GZIP);
  Curl.setopt t (Curl.CURLOPT_FOLLOWLOCATION true);
  Curl.setopt t (Curl.CURLOPT_HTTPHEADER ["Expect:";"X-API-Key: " ^ key]);
  Curl.setopt t (Curl.CURLOPT_URL url);
  Curl.setopt t (Curl.CURLOPT_HTTPGET true);
  Curl.set_writefunction t (writer_callback buff);
  Curl.perform t;
  Curl.cleanup t;
  Unix.sleep 1;
  Buffer.contents buff

let getField fieldName = function
  | `O l -> begin
      try snd (List.find (fun (f,_) -> f = fieldName) l)
      with Not_found -> failwith (Printf.sprintf "field %s not found in json data" fieldName)
    end
  | _ -> failwith "Json structure error"

exception JsonTypeError of string * Ezjsonm.value

let ofJsonFloat = function
  | `Float f -> (f : float)
  | _ -> failwith "This is not a float"

let ofJsonString = function
  | `String s -> (s : string)
  | _ -> failwith "This is not a string"

let ofJsonArray = function
  | `A a -> (a : _ list)
  | _ -> failwith "This is not an array"

let ofJsonInt = function
  | `Float i -> int_of_float i
  | json -> raise (JsonTypeError ("This is not an integer",json))

let check json =
  match getField "ok" json with
  | `Bool true -> ()
  | _    -> failwith "Too many request per second/hour"

let getLastSnapshot () =
  let datas = getIcfpcDatas contest_status in
  let json = Ezjsonm.from_string datas in
  check json;
  let snapshot = match getField "snapshots" json with
    | `A l -> List.rev_map (fun x -> (ofJsonString (getField "snapshot_hash" x),
                                      ofJsonFloat  (getField "snapshot_time" x))) l
    | _ -> failwith "no snapshot list found" in
  (List.hd (List.sort (fun (_,f1) (_,f2) -> compare f2 f1) snapshot))

type problem = {
  publish_time      : float;
  solution_size     : int;
  problem_id        : int;
  owner             : string;
  problem_size      : int;
  problem_spec_hash : string
}

let getProblems () =
  let snapshot,timestamp = getLastSnapshot () in
  let datas = getIcfpcDatas (contest_datas snapshot) in
  let json = Ezjsonm.from_string datas in
  let problems = match getField "problems" json  with
    | `A l -> List.rev_map (fun x -> {
          publish_time      = ofJsonFloat  (getField "publish_time" x);
          solution_size     = ofJsonInt    (getField "solution_size" x);
          problem_id        = ofJsonInt    (getField "problem_id" x);
          owner             = ofJsonString (getField "owner" x);
          problem_size      = ofJsonInt    (getField "problem_size" x);
          problem_spec_hash = ofJsonString (getField "problem_spec_hash" x)}) l
    | _ -> failwith "no problems list found" in
  problems

let filename_of_problem problem =
  Filename.concat problem_directory (Printf.sprintf "%04i.origami" problem.problem_id)

let writeProblem problem =
  let filename = filename_of_problem problem in
  match Sys.file_exists filename with
  | true  -> ()
  | false ->
    let datas = getIcfpcDatas (contest_datas problem.problem_spec_hash) in
    let file = open_out filename in
    output_string file datas

let listProblem = getProblems ()
let result = List.map writeProblem listProblem

