(** *)
open Core.Std

(** {2 To be used by clients} *)
type best = float Int.Table.t

type problem = {
  publish_time      : float;
  solution_size     : int;
  problem_id        : int;
  owner             : string;
  problem_size      : int;
  problem_spec_hash : string;
  ranking: (float * int) list;
} [@@deriving sexp]

val listProblems: ?best:best -> unit -> problem list
(** if best is set don't list problems for which we already have the
    best solution, or are our own *)
val filterProblems: best -> problem list -> problem list
val readProblem: problem -> Icfp_parse.problem
(** get problem *)


type resemblance = [`Good of float | `Unknown | `Perhaps of float] [@@deriving sexp]
val compare_resemblance: resemblance -> resemblance -> int

type solution = {
  problem_id: int;
  solution: Solution.solution;
  resemblance: resemblance;
} [@@deriving sexp]

val saveSolution: ?best:best -> solution -> unit
(** use best if provided, and update it *)

val ourBestSolutions: unit -> best


val save_db: string -> unit
(** save the database, give a name to the modification *)

(** {2 To be used by the server} *)




val helloworld: unit -> bool

type leaderboard = string * float

val getStatus: unit -> (problem list * leaderboard list) option
val writeProblem: problem -> unit

val sendSolution: solution -> [`Ok of (int * float) | `Error of string]
val loadSolutions: unit -> (solution * string) list
val backupSolution: (solution * string * float) -> unit
val badSolution: string -> unit

val update_db: unit -> unit

val foldSolutions_sent: init:'a -> f:('a -> string -> solution -> 'a) -> 'a

val i'm_the_server: unit -> unit


module Convert : sig
  val convert : string -> string -> unit
end
