let split sep s =
  let r = ref [] in
  let j = ref (String.length s) in
  for i = String.length s - 1 downto 0 do
    if String.unsafe_get s i = sep then begin
      r := String.sub s (i + 1) (!j - i - 1) :: !r;
      j := i
    end
  done;
  String.sub s 0 !j :: !r

let lines s =
  let l = split '\n' s in
  let l = List.mapi (fun i v -> (i+1), v) l in
  List.filter (fun (_, l) -> l <> "") l

let get_line s l =
  match l with
  | [] -> failwith (Printf.sprintf "not more lines: %s" s)
  | h :: t ->
    h, t

let parse_int_line s l =
  let (n, line), rem = get_line s l in
  match int_of_string line with
  | exception _ ->
    failwith (Printf.sprintf "line %i is not an int for %s: %s" n s line)
  | i -> i, rem

let number_of_polygons l =
  parse_int_line "polygons" l

let number_of_vertices i l =
  parse_int_line (Printf.sprintf "vertices %i" i) l

let number_of_line_segment l =
  parse_int_line "line segments" l

let get_n_lines n f l =
  let rec loop acc n l =
    if n = 0 then
      List.rev acc, l
    else
      let v, rem = f n l in
      loop (v::acc) (n-1) rem
  in
  loop [] n l

(* let parse_vertices n l = *)
(*   get_n_lines n number_of_vertices l *)

let rationnal_of_string s =
  try Scanf.sscanf s "%[^/]%!" (fun i -> Q.of_string i) with
  | _ ->
    Scanf.sscanf s "%[^/]/%[^/]%!" (fun i j -> Q.make (Z.of_string i) (Z.of_string j))

let vertex_of_string (n, s) =
  match split ',' s with
  | [x; y] -> Geom_ops.{ x = rationnal_of_string x ; y = rationnal_of_string y }
  | _ ->
    failwith (Printf.sprintf "malformed line %i, not a vertex: %s" n s)

let polygon_vertices n l =
  get_n_lines n (fun n l ->
      let v, rem = get_line (Printf.sprintf "vertex %i" n) l in
      vertex_of_string v, rem)
    l

let parse_polygon i l =
  let n, l = number_of_vertices i l in
  polygon_vertices n l

let parse_polygons n l =
  get_n_lines n parse_polygon l

let parse_segment seg_n l =
  let (n, line), rem = get_line "segment" l in
  match split ' ' line with
  | [v1; v2] -> Geom_ops.S (vertex_of_string (n,v1), vertex_of_string (n,v2)), rem
  | _ ->
    failwith (Printf.sprintf "malformed line %i for segment %i, not a segment: %s" n seg_n line)

let parse_segments n l =
  get_n_lines n parse_segment l

let parse_silhouette l =
  let polygons, l = number_of_polygons l in
  let polygones, l = parse_polygons polygons l in
  polygones, l

let parse_skeleton l =
  let segments, l = number_of_line_segment l in
  parse_segments segments l

open Core.Std
type problem = Geom_ops.vertex list list * Geom_ops.segment list
             [@@ deriving sexp]

let parse s : problem =
  let l = lines s in
  let spec, l = parse_silhouette l in
  let skeleton, l = parse_skeleton l in
  match l with
  | [] -> spec, skeleton
  | (n, s) :: _ ->
    failwith (Printf.sprintf "some remaining lines: %n %s" n s)
