open Core.Std


let run best prob =
  try
    let problem = Http_api.readProblem prob in
    let state, _ = Convex_solver.solve_while_not_too_big problem in
    (* let state,_ = Convex_solver.launch_and_replace problem 3 in *)
    let solution = Pliage.to_solution state in
    let solution = Solution.renormalize_solution solution in
    if Solution.check solution ~verbose:() then begin
      Http_api.saveSolution ~best {
        problem_id = prob.problem_id;
        solution;
        resemblance = `Unknown;
      }
    end
    else
      Format.printf "@[Bad solution for is_a_rectangle in solution for prob:%i@]@."
        prob.problem_id
  with _ ->
    Format.printf "@[Raised for prob:%i@]@."
      prob.problem_id

let () =
  let best = Http_api.ourBestSolutions () in
  let problems = Http_api.listProblems ~best () in
  List.iter problems ~f:(run best);
  (* Http_api.save_db "convex solver" *)
