(*

#use "topfind";;
#require "zarith";;
#install_printer Z.pp_print;;

let deux = Z.of_int 2

let triple_simple n =
  let open Z in
  let n = of_int n in
  let a = n * n - one in
  let b = deux * n in
  let c = n * n + one in
  a, b, c

let sin_cos_rat n =
  let open Q in
  let a, b, c = triple_simple n in
  let a, b, c = Q.of_bigint a, Q.of_bigint b, Q.of_bigint c in
  let sin = a / c in
  let cos = b / c in
  sin, cos

let rec (-->) x y =
  if x > y then
    []
  else
    x :: (x+1) --> y

let comp (sin1, cos1) (sin2, cos2) =
  Q.compare Q.(abs (sin1 - cos1)) Q.(abs (sin2 - cos2))

let sins_coss =
  let l = List.map sin_cos_rat (1 --> 1000) in
  let l = List.sort comp l in
  List.map (fun (sin, cos) -> Q.(abs (sin - cos)), sin, cos) l
*)

let z_vec m =
  match Array.map Z.of_int m with
  | [| x; y; z |] -> x, y, z
  | _ -> assert false

let z_mat m =
  match Array.map z_vec m with
  | [| x; y; z |] -> x, y, z
  | _ -> assert false

(* https://en.wikipedia.org/wiki/Tree_of_primitive_Pythagorean_triples *)

(* Et maintenant arrive la magie *)

let origin = z_vec [|3; 4; 5|]

let mat1 =
  z_mat
  [| [| 1; -2; 2 |];
     [| 2; -1; 2 |];
     [| 2; -2; 3 |]; |]

let mat2 =
  z_mat
  [| [| 1; 2; 2 |];
     [| 2; 1; 2 |];
     [| 2; 2; 3 |]; |]

let mat3 =
  z_mat
  [| [| -1; 2; 2 |];
     [| -2; 1; 2 |];
     [| -2; 2; 3 |]; |]

let ( ** ) (a1,a2,a3) (b1,b2,b3) =
  let open Z in
  a1 * b1 + a2 * b2 + a3 * b3

let ( *| ) (v1,v2,v3) o =
  (v1 ** o, v2 ** o, v3 ** o)

let step l =
  List.flatten (List.rev_map (fun v -> [ mat1 *| v; mat2 *| v; mat3 *| v ]) l)

let steps' n orig =
  let rec loop acc n l =
    if n = 0 then
      acc
    else
      let pyt = step l in
      loop (List.rev_append pyt acc) (n-1) pyt
  in
  loop orig n orig

let steps n =
  steps' n [origin]

let deux = Z.of_int 2

let triple_simple n =
  let open Z in
  let n = of_int n in
  let a = n * n - one in
  let b = deux * n in
  let c = n * n + one in
  a, b, c

let rec (-->) x y =
  if x > y then
    []
  else
    x :: (x+1) --> y

let lst =
  let l = List.rev_map triple_simple (2 --> 400) in
  let l' = List.rev_map (fun (a, b, c) -> (b, a, c)) l in
  (* Core.Std.List.dedup *)
  (List.rev_append l l')

let lst = steps' 3 lst
(* let lst = steps' 4 lst *)
(* let lst = steps 11 *)
let len = List.length lst

let sin_cos_lst =
  List.rev_map (fun (a, b, c) ->
      let open Q in
      let a, b, c = Q.of_bigint a, Q.of_bigint b, Q.of_bigint c in
      let sin = a / c in
      let cos = b / c in
      (sin, cos)) lst

(* let sin_cos_lst = *)
(*   let open Q in *)
(*   List.flatten @@ *)
(*   List.rev_map (fun (sin, cos) -> *)
(*       [sin, cos; *)
(*        -sin, cos; *)
(*        sin, -cos; *)
(*        -sin, -cos]) *)
(*       sin_cos_lst *)

let float_of_q q =
  Z.to_float q.Q.num /. Z.to_float q.Q.den

let flst =
  List.sort (fun (s1, _) (s2, _) -> compare s1 s2) @@
  List.rev_map (fun (sin, cos) -> float_of_q sin, float_of_q cos) sin_cos_lst

let pi = 4. *. atan 1.

let angles =
  List.rev_map (fun (s, _) -> 90. *. 2. *. asin s /. pi) flst

let smallest comp lst =
  match lst with
  | [] -> assert false
  | h :: t ->
    List.fold_left (fun acc elt ->
        if comp acc elt < 1 then
          elt
        else
          acc)
      h t

let sin_of_sin_square obj =
  let comp (s2, _) (s1, _) =
    Q.(compare (abs ((s1 * s1) - obj)) (abs ((s2 * s2) - obj)))
  in
  smallest comp sin_cos_lst

(* let dir_of_dir_square (sinsq, cossq) = *)
(*   let comp v1 v2 = *)
(*     let eval (sin, cos) = *)
(*       Q.(abs ((sin * sin) - sinsq) + abs ((cos * cos) - cossq)) *)
(*     in *)
(*     Q.(compare (eval v1) (eval v2)) *)
(*   in *)
(*   smallest comp sin_cos_lst *)


(* let sin_cos ((a, b, c) as t) = *)
(*   let open Q in *)
(*   let a, b, c = Q.of_bigint a, Q.of_bigint b, Q.of_bigint c in *)
(*   let sin = a / c in *)
(*   let cos = b / c in *)
(*   (sin, cos), t *)

(* let comp ((sin1, cos1), _) ((sin2, cos2), _) = *)
(*   Q.compare Q.(abs (sin1 - cos1)) Q.(abs (sin2 - cos2)) *)

(* let sins_coss = *)
(*   let l = List.map sin_cos lst in *)
(*   let l = List.sort comp l in *)
(*   List.map (fun ((sin, cos), t) -> Q.(abs (sin - cos)), sin, cos, t) l *)

(* let (_d, sin, cos, triplet) = List.hd sins_coss *)
(* let _ = assert( Q.(sin * sin + cos * cos) = Q.one) *)

(*
7:

val _d : Q.t = {Q.num = 1; den = 1136689}
val sin : Q.t = {Q.num = 803761; den = 1136689}
val cos : Q.t = {Q.num = 803760; den = 1136689}
val triplet : Z.t * Z.t * Z.t = (803761, 803760, 1136689)

10:
val _d : Q.t = {Q.num = 1; den = 225058681}
val sin : Q.t = {Q.num = 159140519; den = 225058681}
val cos : Q.t = {Q.num = 159140520; den = 225058681}
val triplet : Z.t * Z.t * Z.t = (159140519, 159140520, 225058681)
*)

