open Core.Std
open Http_api


let problem_update_tick = Time.Span.create ~min:5 ()
let nb_not_perfect_send_each_time = 10

let () = i'm_the_server ()

let check_connection () =
  if helloworld ()
  then Format.printf "@[Connection ok.@]@."
  else begin
    Format.printf "@[Connection bad.@]@.";
    exit 1;
  end

let base = Time.now ()

let last_problem_update = ref base
let problem_interest = Int.Table.create ()
let all_problems = ref []

(** each hour *)
let updateProblems () =
  if Time.is_later (Time.now ())
      (Time.next_multiple
         ~can_equal_after:true (** for the first *)
         ~base
         ~after:(!last_problem_update)
         ~interval:problem_update_tick
         ()) then
    match getStatus () with
    | None ->
      Format.printf "@[Connection bad.@]@.";
      exit 1;
    | Some (problems,_) ->
      List.iter ~f:writeProblem problems;
      save_db "problems loaded";
      Int.Table.clear problem_interest;
      List.iter problems ~f:(fun p ->
          let nb_perfect, nb_inperfect =
            List.fold p.ranking ~init:(0,0) ~f:(fun (p,i) (r,_) ->
                if r = 1. then (p+1,i) else (p,i+1)
              ) in
          let badness = (float nb_perfect) *. 100. +. (float nb_inperfect) in
          Int.Table.add_exn problem_interest p.problem_id (-.badness)
        );
      all_problems := problems;
      last_problem_update := Time.now ()

(** initialize problems *)

let sendSolution (s,file) =
  Format.printf "@[SendSolution %s@]@." file;
  match sendSolution s with
  | `Ok (id,resemblance) -> begin
    match s.resemblance with
    | `Good f when resemblance <> f ->
      (Format.printf "@[Error bad resemblance %s s=%f <> %f @]@." file f resemblance;
       false)
    | `Perhaps f when f > resemblance ->
      (Format.printf "@[Error bad resemblance %s s=%f > %f @]@." file f resemblance;
       false)
    | _ -> begin
        backupSolution (s,file,resemblance);
        Format.printf "@[Send succesfull: id=%i ressemblance=%f@]@." id resemblance;
        true
      end
    end
  | `Error err ->
    begin
      (badSolution file;
       Format.printf "@[Error during send of %s : %s@]@." file err;
       false);
    end

let send_some_solutions () =
  let solutions = loadSolutions () in
  let solutions =
    List.sort solutions ~cmp:(fun (sol1,_) (sol2,_) ->
        let c = compare_resemblance sol1.resemblance sol2.resemblance in
        if c <> 0 then c
        else
          let goodness id =
            match Int.Table.find problem_interest id with
            | None ->
              Format.printf "@[Error: unknown problem %i@];@." id;
              0.
            | Some s -> s in
          compare (goodness sol1.problem_id) (goodness sol2.problem_id)
      ) in
  let rec send rem = function
    | [] -> ()
    | ((s,_) as sf)::l when s.resemblance = `Good 1. || s.resemblance = `Perhaps 1. ->
      ignore (sendSolution sf);
      send rem l
    | sf::l ->
      if not (sendSolution sf) then send rem l
      else if 1 < rem then send (rem-1) l
  in
  send nb_not_perfect_send_each_time solutions;
  if solutions <> [] then save_db "solutions send"


let rec run () =
  updateProblems ();
  update_db (); (** takes ~2 sec *)
  send_some_solutions ();
  run ()

let () = run ()
