let input_file filename =
  let f = open_in filename in
  let buf = Buffer.create 1024 in
  let b = Bytes.create 1024 in
  let rec loop () =
    let len = input f b 0 1024 in
    if len = 0 then ()
    else begin
      Buffer.add_subbytes buf b 0 len;
      loop ()
    end
  in
  loop ();
  close_in f;
  Buffer.contents buf

let () =
  let first = 1 in
  let last = Array.length Sys.argv - 1 in
  if last < first then begin
    Format.eprintf "Expecting one or more problem files.@." ;
    exit 2
  end else begin
    Show.init () ;
    let rec loop i =
      let filename = Sys.argv.(i) in
      let problem_text = input_file filename in
      let problem = Icfp_parse.parse problem_text in
      match Show.wait problem
              ("Displaying: " ^ filename)
              ([ 'q', ("quit", `Quit);
                 'd', ("delaunay", `Delaunay);
                 's', ("skeleton", `Skeleton);
                 '+', ("scale +", `Scale(2.));
                 '-', ("scale -", `Scale(0.5));
               ]
               @ (if i < last then [ 'n', ("next", `Next)] else [])
               @ (if i > first then [ 'p', ("prev", `Prev)] else []))
      with
      | `Quit -> ()
      | `Next -> loop (succ i)
      | `Prev -> loop (pred i)
      | `Delaunay -> Show.delaunay := (!Show.delaunay + 1) mod 4; loop i
      | `Skeleton -> Show.skeleton := not !Show.skeleton; loop i
      | `Scale(f) -> Show.scale_user := f *. !Show.scale_user; loop i
    in
    loop first
  end
